/***************************************************************************
 *   Copyright (C) 2006-2014 by Helge Kortendieck  alan@kasmithee.de       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "qaplay.h"


Qaplay::Qaplay(QSystemTrayIcon *parent) : QSystemTrayIcon(parent)
{
	slClArgs = QApplication::arguments();     // get command line args
	sSongCur = slClArgs.at(1);

	QFileInfo qfi(sSongCur);
	if( qfi.exists() == false or qfi.isReadable() == false ) {
		QMessageBox::warning( 0, tr( "Error - file not readable!" ), 
								tr( "The file you specified for playing does either not exist or is not readable!\n"
									"%1\n \n"
									"Please check the name or whether you have read permission for the file.\n \n"
									"The program will exit after you clicked 'OK'..." ).arg( sSongCur ) );
		exit(0);
	}

	setIcon(QIcon("/usr/share/qaplay/icons/scalable/qaplay.svg"));
	setToolTip(tr("Right-click and select 'Quit' to stop playing:\n%1").arg(sSongCur));

	actionsDef();                   // define menu actions

	menu = new QMenu;
	menuCreate();                   // create menu

	readDir();
	bAh = false;
}

Qaplay::~Qaplay()
{
}

void Qaplay::quit()
{
	exit(0);
}


void Qaplay::actionsDef()
{
	actHelp = new QAction( QIcon( "/usr/share/qaplay/icons/scalable/help.svg"), tr("&Help"), this);
	actHelp->setStatusTip(tr("Show Help") );
	connect(actHelp, SIGNAL(triggered()), this, SLOT(ahHelp()));

	actAbout = new QAction(QIcon("/usr/share/qaplay/icons/scalable/about.svg"), tr("&About"), this);
	actAbout->setStatusTip(tr("Show about"));
	connect(actAbout, SIGNAL(triggered()), this, SLOT(ahAbout()));

	actPrev = new QAction(QIcon("/usr/share/qaplay/icons/scalable/prev.svg"), tr("&Previous song"), this);
	actPrev->setStatusTip(tr("Previous song"));
	connect(actPrev, SIGNAL(triggered()), this, SLOT(playPrev()));

	actNext = new QAction(QIcon("/usr/share/qaplay/icons/scalable/next.svg"), tr("&Next song"), this);
	actNext->setStatusTip(tr("Next song"));
	connect(actNext, SIGNAL(triggered()), this, SLOT(playNext()));

	actStop = new QAction(QIcon("/usr/share/qaplay/icons/scalable/stop.svg"), tr("&Stop playing"), this);
	actStop->setStatusTip(tr("Stop playing"));
	connect(actStop, SIGNAL(triggered()), this, SLOT(playStop()));

	actQuit = new QAction(QIcon("/usr/share/qaplay/icons/scalable/quit.svg"), tr("&Quit"), this);
	actQuit->setStatusTip(tr("Quit playing"));
	connect(actQuit, SIGNAL(triggered()), this, SLOT(quit()));

	actInfo = new QAction(QIcon("/usr/share/qaplay/icons/scalable/info.svg"), tr("File/Song I&nfo"), this);
	actInfo->setStatusTip(tr("Show file/song info"));
	connect(actInfo, SIGNAL(triggered()), this, SLOT(showInfo()));

	actFavAdd = new QAction(QIcon("/usr/share/qaplay/icons/scalable/favAdd.svg"), tr("A&dd to Favorites"), this);
	actFavAdd->setStatusTip(tr("Add to favorites list"));
	connect(actFavAdd, SIGNAL(triggered()), this, SLOT(favAdd()));

	actFavPlay = new QAction(QIcon("/usr/share/qaplay/icons/scalable/favPlay.svg"), tr("Pla&y Favorites"), this);
	actFavPlay->setStatusTip(tr("Play favorites list"));
	connect(actFavPlay, SIGNAL(triggered()), this, SLOT(favPlay()));
}


void Qaplay::menuCreate()
{
	menu->clear();
	menu->addAction(QIcon("/usr/share/qaplay/icons/qaplay.png"), tr("qaplay"));
	menu->addSeparator();
	menu->addAction(actHelp);				// show help widget
	menu->addAction(actAbout);				// show about widget
	menu->addSeparator();
	menu->addAction(actPrev);				// play previous song
	menu->addAction(actNext);				// play next song
	menu->addAction(actStop);				// stop playing
	menu->addSeparator();
	menu->addAction(actInfo);				// show file/song info
	menu->addSeparator();
	menu->addAction(actFavAdd);				// add song to favorites list
	menu->addAction(actFavPlay);			// play favorites list
	menu->addSeparator();
	menu->addAction(actQuit);				// quit
	setContextMenu(menu);
}

void Qaplay::toolTipUpd()					// update tooltip with song currently played
{
	setToolTip(tr("Right-click and select 'Quit' to stop playing:\n%1").arg(slSongs.at(iSongCurListIdx)));
}

void Qaplay::readDir()						// get all songs in dir
{
	QFileInfo fi(sSongCur);
	if(fi.isDir() == true) {
		sD = sSongCur;
		bPlayDir = true;
	} else {
		sD = fi.path();
		bPlayDir = false;
	}
	if(sD.right( 1 ) != "/") {
		sD += "/";
	}

	QDir dir( sD );								// song directory, look for songs to play in this dir
		slFilters << "*.mp3" << "*.ogg" << "*.flac" << "*.wav" << "*.wma" << "*.aac";	// more?
		dir.setNameFilters( slFilters );
		dir.setFilter( QDir::Files | QDir::Readable );
		dir.setSorting( QDir::Name | QDir::IgnoreCase );
	slD = dir.entryList();
	slSongs.clear();

	if(slD.count() > 0) {						// found at least 1 song with filter settings
		for(int i = 0; i < slD.count(); ++i) {
			slSongs.append(sD + slD.at(i));
//cout << "readDir() - add songs: " << slSongs.last().toStdString() << endl;                                               // *****  debug  *****
		}
		bD = false;
		if(bPlayDir == false) {
			for(int i = 0; i < slSongs.count(); ++i) {
				if(slSongs.at(i) == sSongCur) {	// song qaplay was called with is among the list of files returned by dir serach
					bD = true;
					iSongCurListIdx = i;
					break;
				}
			}
			if(bD == false) {						// did not found song qaplay was called with in song list
				slSongs.append(sSongCur);			// qaplay was called with different song type we support (and set filters ), let's try anyway, add at end
				iSongCurListIdx = slSongs.count() - 1;
			}
		} else {
			iSongCurListIdx = 0;				// play dir: start with 1st song in dir
		}
	} else {
		slSongs.append(sSongCur);				// qaplay was called with different song type we support (and set filters ), let's try anyway (can now be a dir!)
		iSongCurListIdx = 0;					// qaplay was invoked with this only file in list
	}
	playCreate();
}

void Qaplay::playCreate()
{
	player = new QMediaPlayer;
	connect(player, SIGNAL(mediaStatusChanged(QMediaPlayer::MediaStatus)), this, SLOT(playStatusChanged(QMediaPlayer::MediaStatus)));
	playPlay();
}

void Qaplay::playPlay()
{
	player->setMedia(QUrl::fromLocalFile(slSongs.at(iSongCurListIdx)));
//	player->setVolume(50);
	toolTipUpd();
	player->play();
}

void Qaplay::playStatusChanged(QMediaPlayer::MediaStatus state)
{
	switch( state) {
		case QMediaPlayer::EndOfMedia:
			if(slSongs.count() - 1 > iSongCurListIdx) {
				iSongCurListIdx += 1;
				playPlay();
			} else {							// quit at end of list
				quit();
			}
			break;
		case QMediaPlayer::InvalidMedia:
			showMessage(tr("Invalid Media File!"), tr("Cannot play the file %1").arg(slSongs.at(iSongCurListIdx)),QSystemTrayIcon::Information,3000);
			quit();
			break;
	}
}

void Qaplay::playPrev()
{
	if(iSongCurListIdx > 0) {
		iSongCurListIdx -= 1;
	} else {
		iSongCurListIdx = slSongs.count() - 1;	// roll over, reached begin, start at end of list, last song again...
	}
	player->stop();
	playPlay();
}

void Qaplay::playNext()
{
	if(slSongs.count() - 1 > iSongCurListIdx) {
		iSongCurListIdx += 1;
	} else {
		iSongCurListIdx = 0;					// roll over, reached end, start at 0, first song again...
	}
	player->stop();
	playPlay();
}

void Qaplay::playStop()
{
	player->stop();
	quit();										// for now we stop, as we cannot re-start without playlist !!!!!!!!!!!!!!!!!!!!!!
}


//  *****  show about/help dialog  *****
void Qaplay::ahAbout()						// show about dialog (1)
{
	if( bAh == false) {
		bAh = true;
		ahShow(1);
	}
}

void Qaplay::ahHelp()						// show help dialog (2)
{
	if( bAh == false) {
		bAh = true;
		ahShow(2);
	}
}

void Qaplay::ahShow(int iD)					// show abotu/help dialog
{
	ahw = new aboutHelp();
	connect(this, SIGNAL(sigAh(int)), ahw, SLOT(showAh(int)));
	connect(ahw, SIGNAL(sigDelete()), this, SLOT(ahDelete()));
	emit sigAh(iD);
}

void Qaplay::ahDelete()						// delete about/help dialog
{
	ahw->deleteLater();
	bAh = false;
}

//  *****  show file/song info  *****
void Qaplay::showInfo()						// show song info
{
	siD = getInfo(slSongs.at(iSongCurListIdx));
	sD = "";
	if(siD.sArtist != "") {
		sD = siD.sArtist;
	}
	if(siD.sTitle != "") {
		sD += " - " + siD.sTitle;
	}
	if(siD.sYear != "0") {
		sD += " - " + siD.sYear;
	}
	if(siD.sAlbum != "") {
		sD += " | " + siD.sAlbum;
	}
	if(siD.sTrack != "0") {
		sD += " - " + siD.sTrack;
	}
	if(siD.sGenre != "") {
		sD += " | " + siD.sGenre;
	}
	if(siD.sComment != "") {
		sD += " | " + siD.sComment;
	}
	if(sD != "") {
		showMessage(tr("Song Info"),sD,QSystemTrayIcon::Information,8000);
	}
//cout << "showInfo() : " << sD.toStdString() << endl;                                               // *****  debug  *****	
}


//  *****  favorites  *****
void Qaplay::favAdd()						// add song to favorites list
{
	
}

void Qaplay::favPlay()						// play favorites list
{
	
}

#include "qaplay.moc"
