/***************************************************************************
 *   Copyright (C) 2006-2013 by Helge Kortendieck  alan@kasmithee.de       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _QAPLAY_H_
#define _QAPLAY_H_

#include <QAction>
#include <QApplication>
#include <QDialog>
#include <QDir>
#include <QFileInfo>
#include <QIcon>
#include <QMediaMetaData>
#include <QMediaPlayer>
#include <QMenu>
#include <QMessageBox>
#include <QProcess>
#include <QString>
#include <QStringList>
#include <QSystemTrayIcon>
#include <QUrl>
#include <QVariant>

#include "abouthelp.h"
#include "structs.h"
#include "info.h"

#include <iostream>
using namespace std;


class Qaplay : public QSystemTrayIcon
{
	Q_OBJECT

public:
	Qaplay( QSystemTrayIcon *parent = 0 );
		~Qaplay();

private:
	bool bAh;				// about/help widget is shown
	bool bD;
	bool bPlayDir;			// qaplay was called with a directory, not a song

	int iD;
	int iD2;
	int iSongCurListIdx;	// index of current song in song list

	SongInfo siD;			// song info struct entity

	QAction *actHelp;		// show online help
	QAction *actAbout;		// show about dialog
	QAction *actPrev;		// play previous song
	QAction *actNext;		// play next song
	QAction *actStop;		// stop playing
	QAction *actQuit;		// quit program
	QAction *actInfo;		// show file/song information
	QAction *actFavAdd;		// add song to favorites list
	QAction *actFavPlay;	// play favorites list

	QMediaPlayer *player;

	QMenu *menu;

	QString sD;
QString sD2;			// for debugin only
	QString sSongCur;		// current song (to play)
	QString sQtt;			// qtooltip for the tray icon

	QStringList slClArgs;	// program command line arguments
	QStringList slD;
	QStringList slFilters;	// dir filter to only catch song files
	QStringList slPArgs;	// qprocess command arguments
	QStringList slSongs;	

	QProcess *play;			// qprocess to play song

	QVariant vD;

	aboutHelp *ahw;			// help widget

private slots:
	void ahAbout();
	void ahHelp();
	void ahShow(int iD);
	void ahDelete();

	void quit();

	void actionsDef();
	void menuCreate();

	void favAdd();			// add song to favorites list
	void favPlay();			// play favorites list

	void playCreate();		// create player
	void playPlay();		// play song of song list
	void playStatusChanged(QMediaPlayer::MediaStatus state);
	void playPrev();
	void playNext();
	void playStop();

	void readDir();			// read dir, search for files to play

	void showInfo();		// show song info


	void toolTipUpd();		// update tooltip with song currently played

	signals:
		void sigAh(int iD);
};

#endif	// _QAPLAY_H_
